@@ -11,11 +11,14 @@ from numpy import allclose          as numpy_allclose
 from numpy import array             as numpy_array
 from numpy import ascontiguousarray as numpy_ascontiguousarray 
 from numpy import dtype             as numpy_dtype
+from numpy import isclose           as numpy_isclose
 from numpy import ndarray           as numpy_ndarray
 from numpy import ndim              as numpy_ndim
+from numpy import number            as numpy_number
+from numpy import shape             as numpy_shape
+from numpy import sign              as numpy_sign
 from numpy import take              as numpy_take
 from numpy import tile              as numpy_tile
-from numpy import shape             as numpy_shape
 from numpy import where             as numpy_where
 
 from numpy.ma import all       as numpy_ma_all
@@ -46,8 +49,8 @@ from sys         import executable as sys_executable
 from urlparse    import urlparse as urlparse_urlparse
 from urlparse    import urljoin  as urlparse_urljoin
 
-from .             import __version__, __file__
-from .constants    import CONSTANTS, _file_to_fh
+from .          import __version__, __file__
+from .constants import CONSTANTS, _file_to_fh
 
 # Are we running on GNU/Linux?
 _linux = system() == 'Linux'
@@ -69,7 +72,7 @@ longer required.
 
 :Returns:
 
-    out: `float`
+    out : float
         The amount of available physical memory in bytes.
 
 :Examples:
@@ -112,7 +115,7 @@ The amount of available physical memory.
 
 :Returns:
 
-    out: float
+    out : float
         The amount of available physical memory in bytes.
 
 :Examples:
@@ -143,7 +146,7 @@ process has completed.
 
 :Returns:
 
-    out: `float`
+    out : float
         The amount of free memory in bytes.
 
 :Examples:
@@ -172,11 +175,11 @@ def SET_FREE_MEMORY(*arg):
 
 :Parameters:
 
-    arg: `None`, optional
+    arg : None, optional
 
 :Returns:
 
-    `None`
+    None
 
 :Examples:
 
@@ -197,7 +200,7 @@ The amount of memory which is kept free as a temporary work space.
 
 :Returns:
 
-    out: float
+    out : float
         The amount of memory in bytes.
 
 .. seealso:: `cf.CHUNKSIZE`, `cf.MINNCFM`
@@ -224,13 +227,13 @@ be kept free as a temporary work space is also updated.
 
 :Parameters:
 
-    size: int, optional
+    size : int, optional
         The new chunk size in bytes. The default is to not change the
         current value.
 
 :Returns:
 
-    out: float
+    out : float
         The value prior to the change, or the current value if no new
         value was specified.
 
@@ -248,12 +251,10 @@ be kept free as a temporary work space is also updated.
 
 '''
     old = CONSTANTS['CHUNKSIZE']
- 
     if size:
         size = size[0]
         CONSTANTS['CHUNKSIZE']    = size
         CONSTANTS['FM_THRESHOLD'] = MINNCFM() * size
-    #--- End: if
 
     return old
 #--- End: def
@@ -274,13 +275,13 @@ A chunk of memory is the amount of memory set by `cf.CHUNKSIZE`.
  
 :Parameters:
 
-    arg: int, optional
+    arg : int, optional
         The number of chunks to be kept free as a temporary work
         space. The default is to not change the current value.
 
 :Returns:
 
-    out: int
+    out : int
         The value prior to the change, or the current value if no new
         value was specified.
 
@@ -298,7 +299,6 @@ A chunk of memory is the amount of memory set by `cf.CHUNKSIZE`.
 
 '''
     old = CONSTANTS['MINNCFM']
-
     if arg:
         minncfm = arg[0]
         CONSTANTS['MINNCFM']      = minncfm
@@ -317,7 +317,7 @@ not exist.
 
 :Parameters:
 
-    arg: str, optional
+    arg : str, optional
         The new directory for temporary files. Tilde expansion (an
         initial component of ``~`` or ``~user`` is replaced by that
         *user*'s home directory) and environment variable expansion
@@ -329,7 +329,7 @@ not exist.
 
 :Returns:
 
-    out: str
+    out : str
         The directory prior to the change, or the current directory if
         no new value was specified.
 
@@ -345,7 +345,6 @@ not exist.
 
 '''
     old = CONSTANTS['TEMPDIR']
-
     if arg:
         tempdir = os_path_expanduser(os_path_expandvars(arg[0]))
 
@@ -375,13 +374,13 @@ needed by a variable to access its data array.
 
 :Parameters:
 
-    arg: float, optional
+    arg : float, optional
         The new fraction (between 0.0 and 1.0). The default is to not
         change the current behaviour.
 
 :Returns:
 
-    out: float
+    out : float
         The value prior to the change, or the current value if no new
         value was specified.
 
@@ -410,7 +409,6 @@ follows:
 
 '''
     old = CONSTANTS['OF_FRACTION']
-
     if arg:
         CONSTANTS['OF_FRACTION'] = arg[0]
 
@@ -418,7 +416,7 @@ follows:
 #--- End: def
 
 def REGRID_LOGGING(*arg):
-    """
+    '''
 
 Whether or not to enable ESMPy logging.
 
@@ -426,13 +424,13 @@ If it is logging is performed after every call to ESMPy.
 
 :Parameters:
 
-    arg: bool, optional
+    arg: `bool`, optional
         The new value (either True to enable logging or False to disable it).
         The default is to not change the current behaviour.
 
 :Returns:
 
-    out: bool
+    out: `bool`
         The value prior to the change, or the current value if no new
         value was specified.
 
@@ -445,11 +443,80 @@ False
 >>> cf.REGRID_LOGGING()
 True
 
-    """
+    '''
     old = CONSTANTS['REGRID_LOGGING']
-    
     if arg:
-        CONSTANTS['REGRID_LOGGING'] = arg[0]
+        CONSTANTS['REGRID_LOGGING'] = bool(arg[0])
+    
+    return old
+#--- End:def
+
+def RELAXED_IDENTITIES(*arg):
+    '''
+
+:Parameters:
+
+    arg: `bool`, optional
+      
+:Returns:
+
+    out: `bool`
+        The value prior to the change, or the current value if no new
+        value was specified.
+
+:Examples:
+
+>>> org = cf.RELAXED_IDENTITIES()
+>>> print org
+False
+>>> cf.RELAXED_IDENTITIES(True)
+False
+>>> cf.RELAXED_IDENTITIES()
+True
+>>> cf.RELAXED_IDENTITIES(org)
+True
+>>> cf.RELAXED_IDENTITIES()
+False
+
+    '''
+    old = CONSTANTS['RELAXED_IDENTITIES']
+    if arg:
+        CONSTANTS['RELAXED_IDENTITIES'] = bool(arg[0])
+    
+    return old
+#--- End:def
+
+def IGNORE_IDENTITIES(*arg):
+    '''
+
+:Parameters:
+
+    arg: `bool`, optional
+      
+:Returns:
+
+    out: `bool`
+        The value prior to the change, or the current value if no new
+        value was specified.
+
+:Examples:
+
+>>> org = cf.IGNORE_IDENTITIES()
+>>> print org
+False
+>>> cf.IGNORE_IDENTITIES(True)
+False
+>>> cf.IGNORE_IDENTITIES()
+True
+>>> cf.IGNORE_IDENTITIES(org)
+True
+>>> cf.IGNORE_IDENTITIES()
+False
+
+    '''
+    old = CONSTANTS['IGNORE_IDENTITIES']
+    if arg:
+        CONSTANTS['IGNORE_IDENTITIES'] = bool(arg[0])
     
     return old
 #--- End:def
@@ -465,10 +532,10 @@ Otherwise ``cf.dump(x)`` is equivalent to ``print x``.
 
 :Parameters:
 
-    x:
+    x :
         The object to print.
 
-    kwargs: *optional*
+    kwargs : *optional*
         As for the input variable's `!dump` method, if it has one.
 
 :Returns:
@@ -516,7 +583,7 @@ fraction is retrieved and set with the `OF_FRACTION` function.
 
 :Returns:
 
-    out: bool
+    out : bool
         Whether or not the number of open files exceeds the threshold.
 
 .. seealso:: `cf.close_files`, `cf.close_one_file`, `cf.open_files`
@@ -555,7 +622,7 @@ fraction is retrieved and set with the `OF_FRACTION` function.
 
 :Returns:
 
-    out: bool
+    out : bool
         Whether or not the number of open files exceeds the threshold.
 
 .. seealso:: `cf.close_files`, `cf.close_one_file`, `cf.open_files`
@@ -591,7 +658,7 @@ If there are no appropriate open files then no action is taken.
 
 :Parameters:
 
-    file_format: str, optional
+    file_format : str, optional
         Only close files of the given format. Recognised formats are
         ``'netCDF'`` and ``'PP'``. By default files of any format are
         closed.
@@ -625,7 +692,8 @@ If there are no appropriate open files then no action is taken.
 #---End: def
 
 def close_one_file(file_format=None):
-    '''Close an arbitrary open file containing a sub-array of a data array.
+    '''Close an arbitrary open file containing a sub-array of a data
+array.
 
 By default a file of arbitrary format is closed, but the choice may be
 restricted to files of a particular format.
@@ -637,7 +705,7 @@ If there are no appropriate open files then no action is taken.
 
 :Parameters:
 
-    file_format: str, optional
+    file_format : str, optional
         Only close a file of the given format. Recognised formats are
         ``'netCDF'`` and ``'PP'``. By default a file of any format is
         closed.
@@ -664,7 +732,7 @@ If there are no appropriate open files then no action is taken.
 {'netCDF': {'file1.nc': <netCDF4.Dataset at 0x181bcd0>,
             'file3.nc': <netCDF4.Dataset at 0x1d185e9>}}
 
-    '''
+'''
     if file_format is not None:
         if file_format in _file_to_fh and _file_to_fh[file_format]:
             filename, fh = next(_file_to_fh[file_format].iteritems())
@@ -685,20 +753,20 @@ If there are no appropriate open files then no action is taken.
 def open_files(file_format=None):
     '''
 
-Return the open files containing sub-arrays of data arrays.
+Return the open files containing sub-arrays of master data arrays.
 
 By default all such files are returned, but the selection may be
 restricted to files of a particular format.
 
 :Parameters:
 
-    file_format: str, optional
+    file_format : str, optional
         Only return files of the given format. Recognised formats are
         ``'netCDF'`` and ``'PP'``. By default all files are returned.
 
 :Returns:
 
-    out: dict
+    out : dict
         If *file_format* is set then return a dictionary of file names
         of the specified format and their open file objects. If
         *file_format* is not set then return a dictionary for which
@@ -742,17 +810,17 @@ unchanged to the variable's *name* method.
 
 :Parameters:
 
-    name: str
+    name : str
 
-    x:
+    x :
         The input variable.
 
-    args, kwargs:
+    args, kwargs :
 
 
 :Returns:
 
-    out: 
+    out : 
         A new variable with size 1 axes inserted into the data array.
 
 :Examples:
@@ -776,20 +844,20 @@ between ``a`` and ``b``.
 
 :Parameters:
 
-    a, b: array_like
+    a, b : array_like
         Input arrays to compare.
 
-    atol: float, optional
+    atol : float, optional
         The absolute tolerance for all numerical comparisons, By
         default the value returned by the `ATOL` function is used.
 
-    rtol: float, optional
+    rtol : float, optional
         The relative tolerance for all numerical comparisons, By
         default the value returned by the `RTOL` function is used.
 
 :Returns:
 
-    out: bool
+    out : bool
         Returns True if the arrays are equal, otherwise False.
 
 :Examples:
@@ -836,24 +904,61 @@ True
                 return out
 #--- End: def
 
-def parse_indices(data, indices, cyclic=False):
+def _numpy_isclose(a, b, rtol=None, atol=None):
+    '''Returns a boolean array where two broadcastable arrays are
+element-wise equal within a tolerance.
+
+The tolerance values are positive, typically very small numbers. The
+relative difference (``rtol * abs(b)``) and the absolute difference
+``atol`` are added together to compare against the absolute difference
+between ``a`` and ``b``.
+
+:Parameters:
+
+    a, b: array_like
+        Input arrays to compare.
+
+    atol: `float`, optional
+        The absolute tolerance for all numerical comparisons, By
+        default the value returned by the `ATOL` function is used.
+
+    rtol: `float`, optional
+        The relative tolerance for all numerical comparisons, By
+        default the value returned by the `RTOL` function is used.
+
+:Returns:
+
+    out: `numpy.ndarray`
+
+:Examples:
+
+    '''      
+    try:
+        return numpy_isclose(a, b, rtol=rtol, atol=atol)
+    except (IndexError, NotImplementedError, TypeError):
+        return a == b
+#--- End: def
+
+def parse_indices(shape, indices, cyclic=False, reverse=False, envelope=False):
     '''
 
 :Parameters:
 
-    data: array-like
+    shape: sequence of `ints`
 
-    indices: tuple
+    indices: `tuple` (not a `list`!)
 
 :Returns:
 
-    out: list [, dict]
+    out: `list` [, `dict`]
 
 :Examples:
 
 '''
     parsed_indices = []
     roll           = {}
+    flip           = []
+    compressed_indices = []
 
     if not isinstance(indices, tuple):
         indices = (indices,)
@@ -861,7 +966,7 @@ def parse_indices(data, indices, cyclic=False):
     # Initialize the list of parsed indices as the input indices with any
     # Ellipsis objects expanded
     length = len(indices)
-    n = data.ndim
+    n = len(shape)
     ndim = n
     for index in indices:
         if index is Ellipsis:
@@ -878,7 +983,7 @@ def parse_indices(data, indices, cyclic=False):
 
     if ndim and len_parsed_indices > ndim:
         raise IndexError("Invalid indices %s for array with shape %s" %
-                         (parsed_indices, data.shape))
+                         (parsed_indices, shape))
 
     if len_parsed_indices < ndim:
         parsed_indices.extend([slice(None)]*(ndim-len_parsed_indices))
@@ -898,9 +1003,11 @@ def parse_indices(data, indices, cyclic=False):
 
     #--- End: if
 
-    for i, (index, size) in enumerate(zip(parsed_indices, data.shape)):
 
+    for i, (index, size) in enumerate(zip(parsed_indices, shape)):
+        is_slice = False
         if isinstance(index, slice):            
+            is_slice = True
             start = index.start
             stop  = index.stop
             step  = index.step
@@ -977,7 +1084,7 @@ def parse_indices(data, indices, cyclic=False):
                     (start < stop and step < 0) or
                     (start > stop and step > 0)):
                     raise IndexError("Invalid indices %s for array with shape %s" %
-                                     (parsed_indices, data.shape))
+                                     (parsed_indices, shape))
                 if step < 0 and stop < 0:
                     stop = None
                 index = slice(start, stop, step)
@@ -985,7 +1092,9 @@ def parse_indices(data, indices, cyclic=False):
         elif isinstance(index, (int, long)):
             if index < 0: 
                 index += size
+
             index = slice(index, index+1, 1)
+            is_slice = True
         else:
             convert2positve = True
             if getattr(getattr(index, 'dtype', None), 'kind', None) == 'b':
@@ -995,53 +1104,147 @@ def parse_indices(data, indices, cyclic=False):
                 if index.size != size:
                     raise IndexError(
                         "Invalid indices %s for array with shape %s" %
-                        (parsed_indices, data.shape))
+                        (parsed_indices, shape))
 
                 index = numpy_where(index)[0]
                 convert2positve = False
             #--- End: if
 
-            len_index = len(index)
-            if len_index == 1:                
-                index = index[0]
+            if not numpy_ndim(index):
                 if index < 0:
                     index += size
-                    
+
                 index = slice(index, index+1, 1)
-            elif not len_index:
-                raise IndexError("Invalid indices %s for array with shape %s" %
-                                 (parsed_indices, data.shape))
+                is_slice = True
             else:
-                if convert2positve:
-                    # Convert to non-negative integer numpy array
-                    index = numpy_array(index)
-                    index = numpy_where(index < 0, index+size, index)
-
-                steps = index[1:] - index[:-1]
-                step = steps[0]
-                if step and not (steps - step).any():
-                    # We can replace the list with a slice object
-                    if step > 0:
-                        start, stop = index[0], index[-1]+1
-                    elif step < 0:
-                        start, stop = index[0], index[-1]-1
-                        
-                    if stop < 0:
-                        stop = None
+                len_index = len(index)
+                if len_index == 1:                
+                    index = index[0]
+                    if index < 0:
+                        index += size
+                    
+                    index = slice(index, index+1, 1)
+                    is_slice = True
+                elif len_index:
+                    if convert2positve:
+                        # Convert to non-negative integer numpy array
+                        index = numpy_array(index)
+                        index = numpy_where(index < 0, index+size, index)
+    
+                    steps = index[1:] - index[:-1]
+                    step = steps[0]
+                    if step and not (steps - step).any():
+                        # Replace the numpy array index with a slice
+                        if step > 0:
+                            start, stop = index[0], index[-1]+1
+                        elif step < 0:
+                            start, stop = index[0], index[-1]-1
                             
-                    index = slice(start, stop, step)
+                        if stop < 0:
+                            stop = None
+                                
+                        index = slice(start, stop, step)
+                        is_slice = True
+                    else:
+                        if ((step > 0 and (steps <= 0).any()) or
+                            (step < 0 and (steps >= 0).any()) or
+                            not step):
+                            raise ValueError("Bad index (not strictly monotonic): %s" % index)
+                            
+                        if reverse and step < 0:
+                            # The array is strictly monotoniticall
+                            # decreasing, so reverse it so that it's
+                            # strictly monotonically increasing.  Make
+                            # a note that this dimension will need
+                            # flipping later
+                            index = index[::-1]
+                            flip.append(i)
+                            step = -step
+                        #--- End: if
 
+                        if envelope:
+                            # Create an envelope slice for a parsed
+                            # index of a numpy array of integers
+                            compressed_indices.append(index)
+
+                            step = numpy_sign(step)
+                            if step > 0:
+                                stop = index[-1] + 1
+                            else:
+                                stop = index[-1] - 1
+                                if stop < 0:
+                                    stop = None
+                                    
+                            index = slice(index[0], stop, step)
+                            is_slice = True
                 else:
-                    index = index.tolist()
+                    raise IndexError(
+                        "Invalid indices {0} for array with shape {1}".format(
+                            parsed_indices, shape))                
+            #--- End: if
+        #--- End: if
+        
+        if is_slice:
+            if reverse and index.step < 0:              
+                # If the slice step is negative, then transform
+                # the original slice to a new slice with a
+                # positive step such that the result of the new
+                # slice is the reverse of the result of the
+                # original slice.
+                #
+                # For example, if the original slice is
+                # slice(6,0,-2) then the new slice will be
+                # slice(2,7,2):
+                #
+                # >>> a = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9]
+                # >>> a[slice(6, 0, -2)]
+                # [6, 4, 2]
+                # >>> a[slice(2, 7, 2)]
+                # [2, 4, 6]
+                # a[slice(6, 0, -2)] == list(reversed(a[slice(2, 7, 2)]))
+                # True
+                start, stop, step = index.indices(size)
+                step    *= -1
+                div, mod = divmod(start-stop-1, step)
+                div_step = div*step
+                start   -= div_step
+                stop     = start + div_step + 1
+                
+                index = slice(start, stop, step)
+                flip.append(i)         
+            #--- End: if      
+
+            # If step is greater than one then make sure that
+            # index.stop isn't bigger than it needs to be
+            if cyclic and index.step > 1:
+                start, stop, step = index.indices(size)
+                div, mod = divmod(stop-start-1, step)
+                stop     = start + div*step + 1
+                index    = slice(start, stop, step)
+            #--- End: if      
+
+            # 
+            if envelope: 
+                # Create an envelope slice for a parsed
+                # index of a numpy array of integers                        
+                compressed_indices.append(index)
+                index = slice(start, stop, (1 if reverse else numpy_sign(step)))
         #--- End: if
                     
         parsed_indices[i] = index    
     #--- End: for
 
-    if cyclic:    
-        return parsed_indices, roll
-    else:
+    if not (cyclic or reverse or envelope):
         return parsed_indices
+
+    out = [parsed_indices]
+    if cyclic:
+        out.append(roll)
+    if reverse:
+        out.append(flip)
+    if envelope:
+        out.append(compressed_indices)
+    return out
 #--- End: def
 
 def get_subspace(array, indices):
@@ -1049,9 +1252,9 @@ def get_subspace(array, indices):
 
 :Parameters:
 
-    array: numpy array
+    array : numpy array
 
-    indices: list
+    indices : list
 
 Subset the input numpy array with the given indices. Indexing is similar to
 that of a numpy array. The differences to numpy array indexing are:
@@ -1089,6 +1292,7 @@ indices must contain an index for each dimension of the input array.
         for axis in gg:
             array = take(array, indices[axis], axis=axis)
             indices[axis] = slice(None)
+        #--- End: for
 
         if len_gg < len(indices):
             array = array[tuple(indices)]
@@ -1168,13 +1372,13 @@ equality.
 
 :Parameters:
 
-    arg: int, optional
+    arg : int, optional
         The new value of absolute tolerance. The default is to not
         change the current value.
 
 :Returns:
 
-    out: float
+    out : float
         The value prior to the change, or the current value if no
         new value was specified.
 
@@ -1192,7 +1396,6 @@ equality.
 
 '''
     old = CONSTANTS['ATOL']
-
     if arg:
         CONSTANTS['ATOL'] = arg[0]
 
@@ -1207,13 +1410,13 @@ tolerant equality.
 
 :Parameters:
 
-    arg: int, optional
+    arg : int, optional
         The new value of relative tolerance. The default is to not
         change the current value.
 
 :Returns:
 
-    out: float
+    out : float
         The value prior to the change, or the current value if no
         new value was specified.
 
@@ -1231,7 +1434,6 @@ tolerant equality.
 
 '''
     old = CONSTANTS['RTOL']
-
     if arg:
         CONSTANTS['RTOL'] = arg[0]
 
@@ -1255,25 +1457,25 @@ y)`` is equivalent to ``y.equals(x)``.
     x, y :
         The objects to compare for equality.
 
-    atol: float, optional
+    atol : float, optional
         The absolute tolerance for all numerical comparisons, By
         default the value returned by the `ATOL` function is used.
 
-    rtol: float, optional
+    rtol : float, optional
         The relative tolerance for all numerical comparisons, By
         default the value returned by the `RTOL` function is used.
 
-    ignore_fill_value: bool, optional
+    ignore_fill_value : bool, optional
         If True then `cf.Data` arrays with different fill values are
         considered equal. By default they are considered unequal.
 
-    traceback: bool, optional
+    traceback : bool, optional
         If True then print a traceback highlighting where the two
         objects differ.
 
 :Returns: 
 
-    out: bool
+    out : bool
         Whether or not the two objects are equal.
 
 :Examples:
@@ -1380,24 +1582,24 @@ used, and in this case ``equivalent(x, y)`` is the same as
 
 :Parameters:
 
-    x, y:
+    x, y :
         The objects to compare for equivalence.
 
-    atol: float, optional
+    atol : float, optional
         The absolute tolerance for all numerical comparisons, By
         default the value returned by the `ATOL` function is used.
 
-    rtol: float, optional
+    rtol : float, optional
         The relative tolerance for all numerical comparisons, By
         default the value returned by the `RTOL` function is used.
 
-    traceback: bool, optional
+    traceback : bool, optional
         If True then print a traceback highlighting where the two
         objects differ.
 
 :Returns: 
 
-    out: bool
+    out : bool
         Whether or not the two objects are equivalent.
 
 :Examples:
@@ -1433,9 +1635,15 @@ True
     if atol is None:
         atol = ATOL()
 
-    if hasattr(x, 'equivalent') and callable(x.equivalent):
-        # x has a callable eequivalent method
-        return x.equivalent(y, rtol=rtol, atol=atol, traceback=traceback)
+    eq = getattr(x, 'equivalent', None)
+    if callable(eq):
+        # x has a callable equivalent method
+        return eq(y, rtol=rtol, atol=atol, traceback=traceback)
+
+    eq = getattr(y, 'equivalent', None)
+    if callable(eq):
+        # y has a callable equivalent method
+        return eq(x, rtol=rtol, atol=atol, traceback=traceback)
 
     return equals(x, y, rtol=rtol, atol=atol, ignore_fill_value=True,
                   traceback=traceback)
@@ -1448,14 +1656,14 @@ Return an iterator over an arbitrarily nested sequence.
 
 :Parameters:
 
-    x: scalar or arbitrarily nested sequence
+    x : scalar or arbitrarily nested sequence
         The arbitrarily nested sequence to be flattened. Note that a
         If *x* is a string or a scalar then this is equivalent to
         passing a single element sequence containing *x*.
 
 :Returns:
 
-    out: generator
+    out : generator
         An iterator over flattened sequence.
 
 :Examples:
@@ -1536,14 +1744,14 @@ The pickling is equivalent to::
 
 :Parameters:
 
-    x: 
+    x : 
         The object to be pickled.
 
-    filename: str
+    filename : str
         The name of the file in which to write the pickled
         representation of *x*.
 
-    overwrite: bool, optional
+    overwrite : bool, optional
         If True a pre-existing output file is over written. By default
         an exception is raised if the output file pre-exists.
 
@@ -1601,17 +1809,17 @@ The unpickling is equivalent to::
 
 :Parameters:
 
-    filename: str
+    filename : str
         The name of the file containing the pickled object.
 
 :Returns:
 
-    out: 
+    out : 
         The reconstituted object.
 
 :Raises:
 
-    UnpicklingError:
+    UnpicklingError :
         If the file can not be unpickled. In particular, this might be
         raised when attempting to unpickle fields which were pickled
         with a different, incompatible version of cf.
@@ -1667,12 +1875,12 @@ If a string containing URL is provided then it is returned unchanged.
 
 :Parameters:
 
-    filename: str
+    filename : str
         The name of the file.
 
 :Returns:
 
-    out: str
+    out : str
         The normalized absolutized version of *filename*.
  
 .. seealso:: `cf.dirname`, `cf.pathjoin`, `cf.relpath`
@@ -1709,16 +1917,16 @@ If a string containing URL is provided then it is returned unchanged.
 
 :Parameters:
 
-    filename: str
+    filename : str
         The name of the file.
 
-    start: str, optional
+    start : str, optional
         The start point for the relative path. By default the current
         directoty is used.
 
 :Returns:
 
-    out: str
+    out : str
         The relative path.
 
 .. seealso:: `cf.abspath`, `cf.dirname`, `cf.pathjoin`
@@ -1753,12 +1961,12 @@ including, the last slash (/) is returned.
 
 :Parameters:
 
-    filename: str
+    filename : str
         The name of the file.
 
 :Returns:
 
-    out: str
+    out : str
         The directory name.
 
 .. seealso:: `cf.abspath`, `cf.pathjoin`, `cf.relpath`
@@ -1789,15 +1997,15 @@ If either of the paths is a URL then a URL will be returned
 
 :Parameters:
 
-    path1: str
+    path1 : str
         The first component of the path.
 
-    path2: str
+    path2 : str
         The second component of the path.
 
 :Returns:
 
-    out: str
+    out : str
         The joined paths.
 
 .. seealso:: `cf.abspath`, `cf.dirname`, `cf.relpath`
@@ -1836,12 +2044,12 @@ Python, numpy and cf.
 
 :Parameters:
 
-    array: numpy.ndarray
+    array : numpy.ndarray
         The numpy array to be hashed. May be a masked array.
 
 :Returns:
 
-    out: int
+    out : int
         The hash value
 
 :Examples:
@@ -1909,7 +2117,7 @@ Inspect the attributes of an object.
 
 :Returns: 
 
-    out: str
+    out : str
 
 :Examples:
 
@@ -1948,13 +2156,13 @@ For example, ``a[...] = broadcast_array(a, b.shape)`` is equivalent to
     
 :Parameters:
   
-    a: numpy array-like
+    a : numpy array-like
     
-    shape: tuple
+    shape : tuple
     
 :Returns:
 
-    out: numpy array
+    out : numpy array
     
 :Examples:
 
@@ -2028,20 +2236,20 @@ between ``a`` and ``b``.
 
 :Parameters:
 
-    x, y: array_like
+    x, y : array_like
         Input arrays to compare.
 
-    atol: float, optional
+    atol : float, optional
         The absolute tolerance for all numerical comparisons, By
         default the value returned by the `ATOL` function is used.
 
-    rtol: float, optional
+    rtol : float, optional
         The relative tolerance for all numerical comparisons, By
         default the value returned by the `RTOL` function is used.
 
 :Returns:
 
-    out: bool
+    out : bool
         Returns True if the arrays are equal, otherwise False.
 
 :Examples:
@@ -2068,7 +2276,7 @@ between ``a`` and ``b``.
 
 def _section(o, axes=None, data=False, stop=None, chunks=False,
              min_step=1, **kwargs):
-    """
+    '''
 
 Return a list of m dimensional sections of a Field of n dimensions or
 a dictionary of m dimensional sections of a Data object of n
@@ -2081,7 +2289,7 @@ cf.Data.
 
 :Parameters:
 
-    axes: *optional*
+    axes : *optional*
         In the case of a Field this is a query for the m axes that
         define the sections of the Field as accepted by the Field
         object's axes method.  The keyword arguments are also passed
@@ -2092,29 +2300,29 @@ cf.Data.
         indices of the m axes that define the sections of the Data
         object. If axes is None (the default) all axes are selected.
     
-    data: bool, optional
+    data : bool, optional
         If True this indicates that a data object has been passed, if
         False it indicates that a field object has been passed. By
         default it is False.
 
-    stop: int, optional
+    stop : int, optional
         Stop after taking this number of sections and return. If stop
         is None all sections are taken.
     
-    chunks: bool, optional
+    chunks : bool, optional
         If True return sections that are of the maximum possible size
         that will fit in one chunk of memory instead of sectioning
         into slices of size 1 along the dimensions that are being
         sectioned.
     
-    min_step: int, optional
+    min_step : int, optional
         The minimum step size when making chunks. By default this is
         1. Can be set higher to avoid size 1 dimensions, which are
         problematic for bilinear regridding.
 
 :Returns:
 
-    out: list or dict
+    out : list or dict
         The list of m dimensional sections of the Field or the
         dictionary of m dimensional sections of the Data object.
 
@@ -2131,7 +2339,7 @@ names:
 
 >>> _section(f, ['latitude', 'longitude'], exact=True)
 
-    """
+    '''
     
     # retrieve the index of each axis defining the sections
     if data:
@@ -2200,9 +2408,11 @@ names:
             if data:
                 d[tuple([x.start for x in indices])] = o[tuple(indices)]
             else:
-                fl.append(o.subspace[tuple(indices)])
+                fl.append(o[tuple(indices)])
+
             nl_vars['count'] += 1
             return
+        #--- End: if
         
         if current_index in axis_indices:
             loop_over_index(current_index - 1)
@@ -2213,6 +2423,7 @@ names:
                 return
             indices[current_index] = slice(i, i + steps[current_index])
             loop_over_index(current_index - 1)
+        #--- End: for
     #--- End: def
     
     current_index = len(sizes) - 1
@@ -2242,15 +2453,17 @@ def environment(display=True):
 :Examples:
 
 >>> cf.environment()
-Platform: Linux-3.13.0-24-generic-x86_64-with-debian-jessie-sid
+Platform: Linux-4.4.0-53-generic-x86_64-with-debian-stretch-sid
 HDF5 library: 1.8.17
 netcdf library: 4.4.1
-python: 2.7.12 /home/user/anaconda2/bin/python
-netCDF4: 1.2.4 /home/user/anaconda2/lib/python2.7/site-packages/netCDF4/__init__.pyc
-numpy: 1.11.1 /home/user/anaconda2/lib/python2.7/site-packages/numpy/__init__.pyc
-psutil: 4.3.0 /home/user/anaconda2/lib/python2.7/site-packages/psutil/__init__.pyc
-cfplot: 1.9.21 /home/user/anaconda2/lib/python2.7/site-packages/cfplot/__init__.pyc
-cf: 1.4 /home/user/cf-python/cf/__init__.pyc
+python: 2.7.13 /home/space/anaconda2/bin/python
+netCDF4: 1.2.4 /home/space/anaconda2/lib/python2.7/site-packages/netCDF4/__init__.pyc
+numpy: 1.11.3 /home/space/anaconda2/lib/python2.7/site-packages/numpy/__init__.pyc
+psutil: 5.0.1 /home/space/anaconda2/lib/python2.7/site-packages/psutil/__init__.pyc
+matplotlib: 1.5.1 /home/space/anaconda2/lib/python2.7/site-packages/matplotlib/__init__.pyc
+ESMF: 7.0.0 /home/space/anaconda2/lib/python2.7/site-packages/ESMF/__init__.pyc
+cfplot: 2.1.10 /home/space/anaconda2/lib/python2.7/site-packages/cfplot/__init__.pyc
+cf2: 2.0 /home/space/cf-python/cf/__init__.pyc
 
     '''
     out = []
@@ -2262,21 +2475,28 @@ cf: 1.4 /home/user/cf-python/cf/__init__.pyc
     out.append('numpy: ' + str(numpy__version__) + ' ' + str(os_path_abspath(numpy__file__)))
     out.append('psutil: ' + str(psutil.__version__) + ' ' + str(os_path_abspath(psutil.__file__)))
 
+    try:
+        import matplotlib
+    except:
+        out.append('matplotlib: not available')
+    else:
+        out.append('matplotlib: ' + str(matplotlib.__version__) + ' ' + str(os_path_abspath(matplotlib.__file__)))
+
     try:
         import ESMF
-    except ImportError:
-        pass
+    except:
+        out.append('ESMF: not available')
     else:
         out.append('ESMF: ' + str(ESMF.__version__) + ' ' + str(os_path_abspath(ESMF.__file__)))
 
     try:
         import cfplot
     except ImportError:
-        pass
+        out.append('cfplot: not available')
     else:
         out.append('cfplot: ' + str(cfplot.__version__) + ' ' + str(os_path_abspath(cfplot.__file__)))
             
-    out.append('cf: ' + str(__version__) + ' ' + str(os_path_abspath(__file__)))
+    out.append('cf2: ' + str(__version__) + ' ' + str(os_path_abspath(__file__)))
     
     out = '\n'.join(out)
 
