@@ -1,5 +1,3 @@
-import cf
-import numpy
 import os
 import itertools
 import time
@@ -7,6 +5,10 @@ from operator import mul
 import unittest
 import inspect
 
+import numpy
+
+import cf
+
 def reshape_array(a, axes):
     new_order = [i for i in range(a.ndim) if i not in axes]
     new_order.extend(axes)
@@ -24,45 +26,52 @@ class DataTest(unittest.TestCase):
     chunk_sizes = (17, 34, 300, 100000)[::-1]
     original_chunksize = cf.CHUNKSIZE()
 
-    test_only = ()
-#    test_only = ('NOTHING!!!!!')
+    filename6 = os.path.join(os.path.dirname(os.path.abspath(__file__)),
+                             'test_file2.nc')
+    
+    test_only = []
+#    test_only = ['NOTHING!!!!!']
 #    test_only = ['test_Data__asdatetime__asreftime__isdatetime']
-#    test_only = ('test_Data__collapse_UNWEIGHTED_MASKED',)
-#    test_only = ('test_Data__collapse_UNWEIGHTED_UNMASKED',)
-#    test_only = ('test_Data__collapse_WEIGHTED_UNMASKED',)
-#    test_only = ('test_Data__collapse_WEIGHTED_MASKED',)
-#    test_only = ('test_Data_any',)
-#    test_only = ('test_Data_mean',)
-#    test_only = ('test_Data_sums_of_weights',)
+#    test_only = ['test_Data__collapse_UNWEIGHTED_MASKED']
+#    test_only = ['test_Data__collapse_UNWEIGHTED_UNMASKED']
+#    test_only = ['test_Data__collapse_WEIGHTED_UNMASKED']
+#    test_only = ['test_Data__collapse_WEIGHTED_MASKED']
+#    test_only = ['test_Data_any']
+#    test_only = ['test_Data_mean']
+#    test_only = ['test_Data_sample_size']
+#    test_only = ['test_Data_section']
 #    test_only = ['test_Data_sd_var']
-#    test_only = ('test_Data_sd_var',)
+#    test_only = ['test_Data_sum_of_weights_sum_of_weights2', 'test_Data_sd_var']
 #    test_only = ['test_Data_max_min_sum']
-#    test_only = ('test_Data___setitem__',)
-#    test_only = ('test_Data_ceil', 'test_Data_floor', 'test_Data_trunc', 'test_Data_rint')
-#    test_only = ('test_Data_array', 'test_Data_varray', 'test_Data_dtarray',)
-#    test_only = ('test_dumpd_loadd',)
-#    test_only = ('test_Data_year_month_day_hour_minute_second',)
-#    test_only =  ('test_Data_BINARY_AND_UNARY_OPERATORS',)
-#    test_only =  ('test_Data_clip',)
+#    test_only = ['test_Data_max']
+#    test_only = ['test_Data___setitem__']
+#    test_only = ['test_Data_ceil', 'test_Data_floor', 'test_Data_trunc', 'test_Data_rint']
+#    test_only = ['test_Data_array', 'test_Data_varray', 'test_Data_dtarray']
+#    test_only = ['test_dumpd_loadd']
+#    test_only = ['test_Data_year_month_day_hour_minute_second']
+#    test_only = ['test_Data_BINARY_AND_UNARY_OPERATORS']
+#    test_only = [test_Data_clip']
 
     # Variables for _collapse
     a = numpy.arange(-100, 200., dtype=float).reshape(3, 4, 5, 5) 
-
-    w = numpy.arange(1, 301., dtype=float).reshape(a.shape)
-    w[-1, -1, ...] = w[-1, -1, ...]*2
-    w /= w.min() 
-
-    ones = numpy.ones(a.shape, dtype=float)
     
+    w = numpy.arange(1, 301., dtype=float).reshape(a.shape)
+    
+    w[-1, -1, ...] = w[-1, -1, ...]*2
+    w /= w.min()
+    
+    
+    ones = numpy.ones(a.shape, dtype=float)
+        
     ma = numpy.ma.arange(-100, 200., dtype=float).reshape(3, 4, 5, 5)
     ma[:, 1, 4, 4] = numpy.ma.masked
     ma[0, :, 2, 3] = numpy.ma.masked
     ma[0, 3, :, 3] = numpy.ma.masked
     ma[1, 2, 3, :] = numpy.ma.masked
-
-#    mw = numpy.ma.array(w)
+    
+    
     mw = numpy.ma.array(w, mask=ma.mask)
-
+    
     mones = numpy.ma.array(ones, mask=ma.mask)
    
     axes_combinations = [axes
@@ -113,11 +122,12 @@ class DataTest(unittest.TestCase):
                                         )):
                     n = -n-1
                     for dvalue, avalue in ((n, n), (cf.masked, numpy.ma.masked), (n, n)):
-                        message = "cf.Data[%s, %s]=%s failed" % (j, i, dvalue)
+                        message = "hardmask={}, cf.Data[{}, {}]]={}={} failed".format(hardmask, j, i, dvalue, avalue)
                         d[j, i] = dvalue
                         a[j, i] = avalue
                         self.assertTrue((d.array == a).all() in (True, numpy.ma.masked), message)
-                        self.assertTrue((d.mask.array == numpy.ma.getmaskarray(a)).all(), message)
+                        self.assertTrue((d.mask.array == numpy.ma.getmaskarray(a)).all(), 
+                                        'd.mask.array='+repr(d.mask.array)+'\nnumpy.ma.getmaskarray(a)='+repr(numpy.ma.getmaskarray(a)))
                 #--- End: for
     
                 a = numpy.ma.arange(3000).reshape(50, 60)
@@ -245,28 +255,31 @@ class DataTest(unittest.TestCase):
             return
 
         # Scalar array
-        for d in [cf.Data(11292.5, 'days since 1970-1-1'),
-                  cf.Data('2000-12-1 12:00', dt=True)]:
+        for d, x in zip([cf.Data(11292.5, 'days since 1970-1-1'),
+                         cf.Data('2000-12-1 12:00', dt=True)],
+                        [11292.5, 0]):
             a = d.dtarray
             self.assertTrue(a.shape == ())
             self.assertTrue(a == numpy.array(cf.dt('2000-12-1 12:00')))
             a = d.array
             self.assertTrue(a.shape == ())
-            self.assertTrue(a == 11292.5)
+            self.assertTrue(a == x)
             a = d.dtarray
             a = d.array
             self.assertTrue(a.shape == ())
-            self.assertTrue(a == 11292.5)
+            self.assertTrue(a == x)
 
         # Non-scalar array
-        for d in [cf.Data([[11292.5, 11293.5]], 'days since 1970-1-1'),
-                  cf.Data([['2000-12-1 12:00', '2000-12-2 12:00']], dt=True)]:
+        for d, x in zip([cf.Data([[11292.5, 11293.5]], 'days since 1970-1-1'),
+                         cf.Data([['2000-12-1 12:00', '2000-12-2 12:00']], dt=True)],
+                        ([[11292.5, 11293.5]],
+                         [[0, 1]])):
             a = d.dtarray
             a = d.array
-            self.assertTrue((a == [[11292.5, 11293.5]]).all())
+            self.assertTrue((a == x).all())
             a = d.dtarray
             a = d.array
-            self.assertTrue((a == [[11292.5, 11293.5]]).all())
+            self.assertTrue((a == x).all())
             a = d.dtarray
             self.assertTrue((a == numpy.array([[cf.dt('2000-12-1 12:00'),
                                                 cf.dt('2000-12-2 12:00')]])).all())
@@ -466,7 +479,6 @@ class DataTest(unittest.TestCase):
                 self.assertTrue(d.max(_preserve_partitions=pp) == 5)
                 self.assertTrue(d.max(_preserve_partitions=pp).datum() == 5)
                 self.assertTrue(d.max(_preserve_partitions=pp) == cf.Data(0.005, 'km'))
-#               print 'pmshape =', d._pmshape
         #--- End: for
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
@@ -568,11 +580,12 @@ class DataTest(unittest.TestCase):
                 for j in range(-a.ndim, a.ndim):
                     b = numpy.swapaxes(a.copy(), i, j)
                     e = d.swapaxes(i, j)
-                    message = 'cf.Data.swapaxes(%d, %d) failed' % (i, j)
+                    message = 'cf.Data.swapaxes({}, {}) failed'.format(i, j)
                     self.assertTrue(b.shape == e.shape, message)
                     self.assertTrue((b == e.array).all(), message)
                 #--- End: for
             #--- End: for
+        #--- End: for
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
                 
@@ -674,9 +687,9 @@ class DataTest(unittest.TestCase):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
 
-        for chunksize in self.chunk_sizes:   
+        for chunksize in self.chunk_sizes:
             cf.CHUNKSIZE(chunksize)          
-            array=numpy.arange(3*4*5).reshape(3, 4, 5) + 1
+            array = numpy.arange(3*4*5).reshape(3, 4, 5) + 1
             
             arrays =  (numpy.arange(3*4*5).reshape(3, 4, 5) + 1.0,
                        numpy.arange(3*4*5).reshape(3, 4, 5) + 1)
@@ -686,59 +699,74 @@ class DataTest(unittest.TestCase):
                     d = cf.Data(a0[(slice(None, None, -1),)*a0.ndim], 'metre')
                     d.flip(i=True)
                     x = cf.Data(a1, 'metre')
-    
-                    self.assertTrue((d +  x).equals(cf.Data(a0 +  a1, 'm' ), traceback=1)), '%s+%s'  % (repr(d), x)
-                    self.assertTrue((d *  x).equals(cf.Data(a0 *  a1, 'm2'), traceback=1)), '%s*%s'  % (repr(d), x)
-                    self.assertTrue((d /  x).equals(cf.Data(a0 /  a1, '1' ), traceback=1)), '%s/%s'  % (repr(d), x)
-                    self.assertTrue((d -  x).equals(cf.Data(a0 -  a1, 'm' ), traceback=1)), '%s-%s'  % (repr(d), x)
-                    self.assertTrue((d // x).equals(cf.Data(a0 // a1, '1' ), traceback=1)), '%s//%s' % (repr(d), x)
-                
-                    self.assertTrue(d.__truediv__(x).equals(cf.Data(array.__truediv__(array), '1'), traceback=1)), '%s.__truediv__(%s)' % (d, x)
-                    self.assertTrue(d.__rtruediv__(x).equals(cf.Data(array.__rtruediv__(array), '1'), traceback=1)) , '%s.__rtruediv__(%s)' % (d, x)
+                    
+                    message = 'Failed in {!r}+{!r}'.format(d, x)
+                    self.assertTrue((d +  x).equals(cf.Data(a0 +  a1, 'm' ), traceback=1), message)
+                    message = 'Failed in {!r}*{!r}'.format(d, x)
+                    self.assertTrue((d *  x).equals(cf.Data(a0 *  a1, 'm2'), traceback=1), message)
+                    message = 'Failed in {!r}/{!r}'.format(d, x)
+                    self.assertTrue((d /  x).equals(cf.Data(a0 /  a1, '1' ), traceback=1), message)
+                    message = 'Failed in {!r}-{!r}'.format(d, x)
+                    self.assertTrue((d -  x).equals(cf.Data(a0 -  a1, 'm' ), traceback=1), message)
+                    message = 'Failed in {!r}//{!r}'.format(d, x)
+                    self.assertTrue((d // x).equals(cf.Data(a0 // a1, '1' ), traceback=1), message)
+
+                    message = 'Failed in {!r}.__truediv__//{!r}'.format(d, x)
+                    self.assertTrue(d.__truediv__(x).equals(cf.Data(array.__truediv__(array), '1'), traceback=1), message)
+
+                    message = 'Failed in {!r}__rtruediv__{!r}'.format(d, x)
+                    self.assertTrue(d.__rtruediv__(x).equals(cf.Data(array.__rtruediv__(array), '1'), traceback=1) , message)
                     
                     try:
                         d ** x
                     except:
                         pass
                     else:
-                        self.assertTrue((d**x).all(), '%s**%s' % (d, repr(x)))
-    
-#                    print '%s {+, -, *, /, //, **, __truediv__, __rtruediv__} %s passed' % (d, x),     "pmshape =", d._pmshape
+                        message = 'Failed in {!r}**{!r}'.format(d, x)
+                        self.assertTrue((d**x).all(), message)
                 #--- End: for                       
             #--- End: for                       
-                       
+                      
             for a0 in arrays:
                 d = cf.Data(a0, 'metre')
-    
-                for x in (2, 2.0):
-                    self.assertTrue((d +  x).equals(cf.Data(a0 +  x, 'm' ), traceback=1)), '%s+%s'  % (repr(d), x)
-                    self.assertTrue((d *  x).equals(cf.Data(a0 *  x, 'm' ), traceback=1)), '%s*%s'  % (repr(d), x)
-                    self.assertTrue((d /  x).equals(cf.Data(a0 /  x, 'm' ), traceback=1)), '%s/%s'  % (repr(d), x)
-                    self.assertTrue((d -  x).equals(cf.Data(a0 -  x, 'm' ), traceback=1)), '%s-%s'  % (repr(d), x)
-                    self.assertTrue((d // x).equals(cf.Data(a0 // x, 'm' ), traceback=1)), '%s//%s' % (repr(d), x)
-                    self.assertTrue((d ** x).equals(cf.Data(a0 ** x, 'm2'), traceback=1)), '%s**%s' % (repr(d), x)
+                for x in (2, 2.0,):
+                    message = 'Failed in {!r}+{}'.format(d, x)
+                    self.assertTrue((d +  x).equals(cf.Data(a0 +  x, 'm' ), traceback=1), message)
+                    message = 'Failed in {!r}*{}'.format(d, x) 
+                    self.assertTrue((d *  x).equals(cf.Data(a0 *  x, 'm' ), traceback=1), message)
+                    message = 'Failed in {!r}/{}'.format(d, x) 
+                    self.assertTrue((d /  x).equals(cf.Data(a0 /  x, 'm' ), traceback=1), message)
+                    message = 'Failed in {!r}-{}'.format(d, x) 
+                    self.assertTrue((d -  x).equals(cf.Data(a0 -  x, 'm' ), traceback=1), message)
+                    message = 'Failed in {!r}//{}'.format(d, x)
+                    self.assertTrue((d // x).equals(cf.Data(a0 // x, 'm' ), traceback=1), message)
+                    message = 'Failed in {!r}**{}'.format(d, x)
+                    self.assertTrue((d ** x).equals(cf.Data(a0 ** x, 'm2'), traceback=1), message)
+                    message = 'Failed in {!r}.__truediv__{}'.format(d, x)
+                    self.assertTrue(d.__truediv__(x).equals(cf.Data(a0.__truediv__(x), 'm'), traceback=1), message)
+                    message = 'Failed in {!r}.__rtruediv__{}'.format(d, x)
+                    self.assertTrue(d.__rtruediv__(x).equals(cf.Data(a0.__rtruediv__(x), 'm-1'), traceback=1), message)
                     
-                    self.assertTrue(d.__truediv__(x).equals(cf.Data(a0.__truediv__(x), 'm'), traceback=1)), '%s.__truediv__(%s)' % (d, x)
-
-                    self.assertTrue(d.__rtruediv__(x).equals(cf.Data(a0.__rtruediv__(x), 'm-1'), traceback=1)) , '%s.__rtruediv__(%s)' % (d, x)
-          
-#                    print '%s {+, -, *, /, //, **, __truediv__, __rtruediv__} %s passed' % (d, x),                         "pmshape =", d._pmshape
+                    message = 'Failed in {}+{!r}'.format(x, d)
+                    self.assertTrue((x +  d).equals(cf.Data(x +  a0, 'm'  ), traceback=1), message)
+                    message = 'Failed in {}*{!r}'.format(x, d)
+                    self.assertTrue((x *  d).equals(cf.Data(x *  a0, 'm'  ), traceback=1), message)
+                    message = 'Failed in {}/{!r}'.format(x, d)
+                    self.assertTrue((x /  d).equals(cf.Data(x /  a0, 'm-1'), traceback=1), message)
+                    message = 'Failed in {}-{!r}'.format(x, d)
+                    self.assertTrue((x -  d).equals(cf.Data(x -  a0, 'm'  ), traceback=1), message)
+                    message = 'Failed in {}//{!r}\n{!r}\n{!r}'.format(x, d, x//d, x//a0)
+                    self.assertTrue((x // d).equals(cf.Data(x // a0, 'm-1'), traceback=1), message)
                     
-                    self.assertTrue((x +  d).equals(cf.Data(x +  a0, 'm'  ), traceback=1)), '%s+%s'  % (x, repr(d))
-                    self.assertTrue((x *  d).equals(cf.Data(x *  a0, 'm'  ), traceback=1)), '%s*%s'  % (x, repr(d))
-                    self.assertTrue((x /  d).equals(cf.Data(x /  a0, 'm-1'), traceback=1)), '%s/%s'  % (x, repr(d))
-                    self.assertTrue((x -  d).equals(cf.Data(x -  a0, 'm'  ), traceback=1)), '%s-%s'  % (x, repr(d))
-                    self.assertTrue((x // d).equals(cf.Data(x // a0, 'm-1'), traceback=1)), '%s//%s' % (x, repr(d))
-    
                     try:
                         x ** d
                     except:
                         pass
                     else:
-                        self.assertTrue((x**d).all(), '%s**%s' % (x, repr(d)))
-    
-#                    print '%s {+, -, *, /, //, **} %s passed' % (x, repr(d)), "pmshape =", d._pmshape
-                
+                        message = 'Failed in {}**{!r}'.format(x, d)
+                        self.assertTrue((x**d).all(), message)
+
+#                    print 'START ====================', repr(d)
                     a = a0.copy()                        
                     try:
                         a += x
@@ -746,8 +774,12 @@ class DataTest(unittest.TestCase):
                         pass
                     else:
                         e = d.copy()
+#                        print '0 ------'
                         e += x
-                        self.assertTrue(e.equals(cf.Data(a, 'm'), traceback=1)), '%s+=%s' % (repr(d), x)
+#                        print '1 ------'
+                        message = 'Failed in {!r}+={}'.format(d, x)
+                        self.assertTrue(e.equals(cf.Data(a, 'm'), traceback=1), message)
+#                    print 'END ====================', repr(d)
 
                     a = a0.copy()                        
                     try:
@@ -789,7 +821,6 @@ class DataTest(unittest.TestCase):
                         e //= x
                         self.assertTrue(e.equals(cf.Data(a, 'm'), traceback=1)), '%s//=%s' % (repr(d), x)
 
-
                     a = a0.copy()                        
                     try:
                         a **= x
@@ -809,8 +840,6 @@ class DataTest(unittest.TestCase):
                         e = d.copy()
                         e.__itruediv__(x)
                         self.assertTrue(e.equals(cf.Data(a, 'm'), traceback=1)), '%s.__itruediv__(%s)' % (d, x)
-                
-#                    print '%s {+=, -=, *=, /=, //=, **=, __itruediv__} %s passed' % (repr(d), x),                         "pmshape =", d._pmshape
                 #--- End: for
                 
                 for x in (cf.Data(2, 'metre'), cf.Data(2.0, 'metre')):
@@ -828,8 +857,6 @@ class DataTest(unittest.TestCase):
                         self.assertTrue((x**d).all(), '%s**%s' % (x, repr(d)))
     
                     self.assertTrue(d.__truediv__(x).equals(cf.Data(a0.__truediv__(x.datum()), ''), traceback=1))
-                
-#                    print '%s {+, -, *, /, //, **, __truediv__} %s passed' % (repr(d), repr(x)),                         "pmshape =", d._pmshape
                 #--- End: for
             #--- End: for
         #--- End: for
@@ -934,9 +961,7 @@ class DataTest(unittest.TestCase):
                     e.shape == shape,
                     "%s, axes=%s, not squeezed bad shape: %s != %s" % \
                     (h, None, e.shape, shape))
-    
-#                print 'h=', h, 'pmshape =', d._pmshape
-            #--- End: for
+                #--- End: for
         #--- End: for
 
         cf.CHUNKSIZE(self.original_chunksize)
@@ -999,6 +1024,21 @@ class DataTest(unittest.TestCase):
         for chunksize in self.chunk_sizes:   
             cf.CHUNKSIZE(chunksize)          
             for pp in (True, False):
+                # unweighted, unmasked
+                d = cf.Data(self.a)
+                for h in ('sum_of_weights', 'sum_of_weights2'):
+                    for axes in self.axes_combinations:
+                        b = reshape_array(self.ones, axes)
+                        b = b.sum(axis=-1)
+                        e = getattr(d, h)(axes=axes, squeeze=True,
+                                          _preserve_partitions=pp)
+                        
+                        self.assertTrue(
+                            e.allclose(b, rtol=1e-05, atol=1e-08),
+                            "%s, axis=%s, unweighted, unmasked, pp=%s, \ne=%s, \nb=%s, \ne-b=%s" % \
+                            (h, axes, pp, e.array, b, e.array-b))
+                #--- End: for
+
                 # unweighted, masked
                 d = cf.Data(self.ma)
                 for a, h in zip((self.mones      , self.mones), 
@@ -1011,32 +1051,18 @@ class DataTest(unittest.TestCase):
         
                         self.assertTrue(
                             (e.mask.array == b.mask).all(),
-                            "%s, axis=%s, \ne.mask=%s, \nb.mask=%s, \ne.mask==b.mask=%s" %
-                            (h, axes, e.mask.array, b.mask, e.mask.array==b.mask))
+                            "%s, axis=%s, unweighted, masked, pp=%s, \ne.mask=%s, \nb.mask=%s, \ne.mask==b.mask=%s" %
+                            (h, axes, pp, e.mask.array, b.mask, e.mask.array==b.mask))
     
                         self.assertTrue(
                             e.allclose(b, rtol=1e-05, atol=1e-08),
-                            "%s, axis=%s, \ne=%s, \nb=%s, \ne-b=%s" %
-                            (h, axes, e.array, b, e.array-b))
+                            "%s, axis=%s, unweighted, masked, pp=%s, \ne=%s, \nb=%s, \ne-b=%s" %
+                            (h, axes, pp, e.array, b, e.array-b))
                     #--- End: for
     #                print "pmshape =", d._pmshape
                 #--- End: for    
     
-                # unweighted, masked
-                d = cf.Data(self.a)
-                for h in ('sum_of_weights', 'sum_of_weights2'):
-                    for axes in self.axes_combinations:
-                        b = reshape_array(self.ones, axes)
-                        b = b.sum(axis=-1)
-                        e = getattr(d, h)(axes=axes, squeeze=True,
-                                          _preserve_partitions=pp)
-                        
-                        self.assertTrue(
-                            e.allclose(b, rtol=1e-05, atol=1e-08),
-                            "%s, axis=%s, \ne=%s, \nb=%s, \ne-b=%s" % \
-                            (h, axes, e.array, b, e.array-b))
-                #--- End: for
-    
+        
                 # weighted, masked
                 d = cf.Data(self.ma)
                 x = cf.Data(self.w)
@@ -1198,70 +1224,12 @@ class DataTest(unittest.TestCase):
         if self.test_only and inspect.stack()[0][3] not in self.test_only:
             return
 
-        ddofs=(0, 1) #(0, 1, 2)
-#        f = 2
+        ddofs=(0, 1)
 
         for chunksize in self.chunk_sizes:   
             for pp in (False, True):
                 cf.CHUNKSIZE(chunksize) 
 
-#                # unweighted, unmasked
-#                d = cf.Data(self.a, units='K')
-#                for np, h in zip((numpy.var, numpy.std),
-#                                 ('var'    , 'sd')):
-#                    for ddof in ddofs:
-#                        for axes in self.axes_combinations:
-#                            b = reshape_array(self.a, axes)
-#                            b = np(b, axis=-1, ddof=ddof)                
-#                            e = getattr(d, h)(axes=axes, squeeze=True,
-#                                              ddof=ddof, _preserve_partitions=pp)
-#                            self.assertTrue(
-#                                e.allclose(b, rtol=1e-05, atol=1e-08),
-#                                "%s, axis=%s, unweighted, unmasked \ne=%s, \nb=%s, \ne-b=%s" %
-#                                (h, axes, e.array, b, e.array-b))
-#                #--- End: for
-#    
-#                # unweighted, masked
-#                d = cf.Data(self.a, units='K')
-#                x = cf.Data(self.w)
-#                for h in ('var', 'sd'):
-#                    for axes in self.axes_combinations:                
-#                        for ddof in ddofs:
-#                            e = getattr(d, h)(axes=axes, squeeze=True, ddof=ddof)
-#        
-#                            b = reshape_array(self.a, axes)
-#        
-#                            not_enough_data = numpy.ma.count(b, axis=-1) <= ddof
-#        
-#                            avg = numpy.ma.average(b, axis=-1)
-#        
-#                            if numpy.ndim(avg) < b.ndim:
-#                                avg = numpy.ma.expand_dims(avg, -1)
-#        
-#                            b, sv = numpy.ma.average((b-avg)**2, axis=-1, returned=True)
-#        
-#                            b = numpy.ma.where(not_enough_data, numpy.ma.masked, b)
-#        
-#                            b *= sv/(sv-ddof)
-#                            if h == 'sd':
-#                                b **= 0.5
-#                            b = numpy.ma.asanyarray(b)
-#        
-#                            e = getattr(d, h)(axes=axes, squeeze=True,
-#                                              ddof=ddof,
-#                                              _preserve_partitions=pp)
-#                            
-#                            self.assertTrue(
-#                                (e.mask.array == b.mask).all(),
-#                                "%s, axes=%s, ddof=%s, \ne.mask=%s, \nb.mask=%s, \ne.mask==b.mask=%s" %
-#                                (h, axes, ddof, e.mask.array, b.mask, e.mask.array==b.mask))
-#                            self.assertTrue(
-#                                e.allclose(b, rtol=1e-05, atol=1e-08),
-#                                "%s, axes=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" %
-#                                (h, axes, ddof, e.array, b, e.array-b))
-#                #--- End: for
-    
-
                 # unweighted, unmasked
                 d = cf.Data(self.a, units='K')
                 for np, h in zip((numpy.var, numpy.std),
@@ -1395,89 +1363,6 @@ class DataTest(unittest.TestCase):
                 #--- End: for
             #--- End: for
         #--- End: for
-
-
-
-#                # weighted, unmasked 
-#                f= 2
-#                d = cf.Data(self.a, units='K')
-#                x = cf.Data(self.w)
-#                for h in ('var', 'sd'):
-#                    for axes in self.axes_combinations:                
-#                        for ddof in ddofs:
-#                            b = reshape_array(self.a, axes)
-#                            v = reshape_array(self.w, axes)
-#                            
-#                            avg = numpy.average(b, axis=-1, weights=v)
-#                            if numpy.ndim(avg) < b.ndim:
-#                                avg = numpy.expand_dims(avg, -1)
-#            
-#                            b, sv = numpy.average((b-avg)**2, axis=-1, weights=v,
-#                                                  returned=True)
-#                            b *= f*sv/(f*sv-ddof)
-#                            
-#                            if h == 'sd':
-#                                b **= 0.5
-#        
-#                            e = getattr(d, h)(axes=axes, weights=x,
-#                                              squeeze=True, ddof=ddof,
-##                                              a=f,
-#                                              _preserve_partitions=pp)
-#        
-#                            self.assertTrue(
-#                                e.allclose(b, rtol=1e-05, atol=1e-08) ,
-#                                "%s, axis=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" % \
-#                                (h, axes, ddof, e.array, b, e.array-b))
-#                #--- End: for
-#    
-#                # weighted, masked
-#                f = 2
-#                d = cf.Data(self.ma, units='K')
-#                x = cf.Data(self.w)
-#                for h in ('var', 'sd'):
-#                    for axes in self.axes_combinations:                
-#                        for ddof in ddofs:
-#                            b = reshape_array(self.ma, axes)
-#                            v = reshape_array(self.mw, axes)
-#        
-#                            not_enough_data = numpy.ma.count(b, axis=-1) <= ddof
-#        
-#                            avg = numpy.ma.average(b, axis=-1, weights=v)
-#                            if numpy.ndim(avg) < b.ndim:
-#                                avg = numpy.ma.expand_dims(avg, -1)
-#        
-#                            b, sv = numpy.ma.average((b-avg)**2, axis=-1, weights=v,
-#                                                     returned=True)
-#        
-#                            b = numpy.ma.where(not_enough_data, numpy.ma.masked, b)
-#        
-#                            b *= f*sv/(f*sv-ddof)
-#                            if h == 'sd':
-#                                b **= 0.5
-#                            b = numpy.ma.asanyarray(b)
-#        
-#                            e = getattr(d, h)(axes=axes, weights=x,
-#                                              squeeze=True, ddof=ddof,
-##                                              a=f,
-#                                              _preserve_partitions=pp)
-#        
-#                            if h == 'sd':
-#                                self.assertTrue(e.Units == d.Units)
-#                            else:                            
-#                                self.assertTrue(e.Units == d.Units**2)
-#                       
-#                            self.assertTrue(
-#                                (e.mask.array == b.mask).all(),
-#                                "%s, axis=%s, \ne.mask=%s, \nb.mask=%s, \ne.mask==b.mask=%s" %
-#                                (h, axes, e.mask.array, b.mask, e.mask.array==b.mask))
-#    
-#                            self.assertTrue(
-#                                e.allclose(b, rtol=1e-05, atol=1e-08),
-#                                "%s, axis=%s, ddof=%s, \ne=%s, \nb=%s, \ne-b=%s" %
-#                                (h, axes, ddof, e.array, b, e.array-b))
-#                #--- End: for
-#            #--- End: for
-#        #--- End: for
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
 
@@ -1488,7 +1373,7 @@ class DataTest(unittest.TestCase):
         for chunksize in self.chunk_sizes:   
             cf.CHUNKSIZE(chunksize) 
             
-            d = cf.read(self.filename)[0].data
+            d = cf.read1(self.filename).data
             
             dumpd = d.dumpd()
             self.assertTrue(d.equals(cf.Data(loadd=d.dumpd()), traceback=True))
@@ -1499,6 +1384,19 @@ class DataTest(unittest.TestCase):
         cf.CHUNKSIZE(self.original_chunksize)
     #--- End: def
 
+    def test_Data_section(self):
+        if self.test_only and inspect.stack()[0][3] not in self.test_only:
+            return
+
+        for chunksize in (300, 10000, 100000)[::-1]:
+            cf.CHUNKSIZE(chunksize)
+            f = cf.read1(self.filename6)
+            self.assertTrue(list(sorted(f.Data.section((1,2)).keys()))
+                            == [(x, None, None) for x in range(1800)])
+        #--- End: for
+        cf.CHUNKSIZE(self.original_chunksize)
+    #--- End: def
+    
 #--- End: class
 
 
