@@ -1,11 +1,12 @@
 # -*- coding: utf-8 -*-
-
 import os
-import numpy
-import cf
 import unittest
 import inspect
 
+import numpy
+
+import cf
+
 class RegridTest(unittest.TestCase):
     filename1 = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                              'file1.nc')
@@ -47,28 +48,25 @@ class RegridTest(unittest.TestCase):
 
         for chunksize in self.chunk_sizes:
             cf.CHUNKSIZE(chunksize)
-            f1 = cf.read(self.filename1)
-            f2 = cf.read(self.filename2)
-            f3 = cf.read(self.filename3)
+            f1 = cf.read1(self.filename1)
+            f2 = cf.read1(self.filename2)
+            f3 = cf.read1(self.filename3)
             self.assertTrue(f3.equals(f1.regrids(f2), traceback=True),
                             'destination = global Field, CHUNKSIZE = %s' % chunksize)
-            self.assertTrue(f3.equals(f1.regrids(f2, method='conservative'),
-                                      traceback=True),
+            self.assertTrue(f3.equals(f1.regrids(f2, method='conservative'), traceback=True),
                             'destination = global Field, CHUNKSIZE = %s' % chunksize)
             dst = {'longitude': f2.dim('X'), 'latitude': f2.dim('Y')}
             self.assertTrue(f3.equals(f1.regrids(dst, dst_cyclic=True),
                                       traceback=True),
                             'destination = global dict, CHUNKSIZE = %s' % chunksize)
             self.assertTrue(f3.equals(f1.regrids(dst, method='conservative',
-                                                 dst_cyclic=True),
-                                      traceback=True),
+                                                 dst_cyclic=True), traceback=True),
                             'destination = global dict, CHUNKSIZE = %s' % chunksize)
-            f4 = cf.read(self.filename4)
-            f5 = cf.read(self.filename5)
+            f4 = cf.read1(self.filename4)
+            f5 = cf.read1(self.filename5)
             self.assertTrue(f4.equals(f1.regrids(f5), traceback=True),
                             'destination = regional Field, CHUNKSIZE = %s' % chunksize)
-            self.assertTrue(f4.equals(f1.regrids(f5, method='bilinear'),
-                                      traceback=True),
+            self.assertTrue(f4.equals(f1.regrids(f5, method='bilinear'), traceback=True),
                             'destination = regional Field, CHUNKSIZE = %s' % chunksize)
         #--- End: for
         cf.CHUNKSIZE(self.original_chunksize)
@@ -85,15 +83,14 @@ class RegridTest(unittest.TestCase):
 
         for chunksize in self.chunk_sizes:
             cf.CHUNKSIZE(chunksize)
-            f1 = cf.read(self.filename7)
-            f2 = cf.read(self.filename8)
-            f3 = cf.read(self.filename9)
-            self.assertTrue(f3.equals(f1.regridc(f2, axes='T', method='bilinear'),
-                                      traceback=True),
+            f1 = cf.read1(self.filename7)
+            f2 = cf.read1(self.filename8)
+            f3 = cf.read1(self.filename9)
+            self.assertTrue(f3.equals(f1.regridc(f2, axes='T', method='bilinear'), traceback=True),
                             'destination = time series, CHUNKSIZE = %s' % chunksize)
-            f4 = cf.read(self.filename1)
-            f5 = cf.read(self.filename2)
-            f6 = cf.read(self.filename10)
+            f4 = cf.read1(self.filename1)
+            f5 = cf.read1(self.filename2)
+            f6 = cf.read1(self.filename10)
             self.assertTrue(f6.equals(f4.regridc(f5, axes=('X', 'Y')),
                                       traceback=True),
                             'destination = global Field, CHUNKSIZE = %s' % chunksize)
@@ -112,32 +109,6 @@ class RegridTest(unittest.TestCase):
 
         cf.ATOL(original_atol)
     #--- End: def
-
-    def test_Field_section(self):
-        if self.test_only and inspect.stack()[0][3] not in self.test_only:
-            return
-
-        for chunksize in self.chunk_sizes:
-            cf.CHUNKSIZE(chunksize)
-            f = cf.read(self.filename6)
-            self.assertTrue(len(f.section(('X','Y'))) == 1800,
-                            'CHUNKSIZE = %s' % chunksize)
-        #--- End: for
-        cf.CHUNKSIZE(self.original_chunksize)
-    #--- End: def
-    
-    def test_Data_section(self):
-        if self.test_only and inspect.stack()[0][3] not in self.test_only:
-            return
-
-        for chunksize in self.chunk_sizes:
-            cf.CHUNKSIZE(chunksize)
-            f = cf.read(self.filename6)
-            self.assertTrue(list(sorted(f.Data.section((1,2)).keys()))
-                            == [(x, None, None) for x in range(1800)])
-        #--- End: for
-        cf.CHUNKSIZE(self.original_chunksize)
-    #--- End: def
     
 #--- End: class
 
